import gi.*;

class Arithmetic extends SLR1_Grammar {

	Arithmetic() {

		/**
		* Lexical Specification
		*/
		int INFINITY = -1;

		put("LITERAL", new Repetition(PosixClass.digit(), 1, INFINITY)); // [[:digit:]]+
		put("SPACE", new Repetition(PosixClass.space(), 1, INFINITY)); // [[:space:]]+

		/**
		* Semantic Specification
		*/
		Semantics print = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println(tree.phrase[0].attribute);
			}
		};
		Semantics add = new Semantics() {
			public void evaluate(ParseTree tree) {
				int left = ((Integer)tree.phrase[0].attribute).intValue();
				int right = ((Integer)tree.phrase[2].attribute).intValue();
				tree.attribute = new Integer(left + right);
			}
		};
		Semantics multiply = new Semantics() {
			public void evaluate(ParseTree tree) {
				int left = ((Integer)tree.phrase[0].attribute).intValue();
				int right = ((Integer)tree.phrase[2].attribute).intValue();
				tree.attribute = new Integer(left * right);
			}
		};
		Semantics decode = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = Integer.decode((String)tree.phrase[0].attribute);
			}
		};
		Semantics second = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = tree.phrase[1].attribute;
			}
		};
		Semantics first = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = tree.phrase[0].attribute;
			}
		};

		/**
		* Syntax Specification
		*/
		put("Start", new Object[][] {
			{"Expression", print}
		});
		put("Expression", new Object[][] {
			{"Expression", "+", "Term", add},
			{"Term", first}
		});
		put("Term", new Object[][] {
			{"Term", "*", "Factor", multiply},
			{"Factor", first}
		});
		put("Factor", new Object[][] {
			{"LITERAL", decode},
			{"(", "Expression", ")", second}
		});
	}

	public static void main(String[] argv) throws Exception {
		new Arithmetic().interpret(argv);
	}
}
