import gi.*;

class ArithmeticLL extends LL1_Grammar {

	ArithmeticLL() {

		/**
		* Lexical Specification
		*/
		int INFINITY = -1;

		put("LITERAL", new Repetition(PosixClass.digit(), 1, INFINITY)); // [[:digit:]]+
		put("SPACE", new Repetition(PosixClass.space(), 1, INFINITY)); // [[:space:]]+

		/**
		* Semantic Specification
		*/
		Semantics print = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println(tree.phrase[0].attribute);
			}
		};
		Semantics add = new Semantics() {
			public void evaluate(ParseTree tree) {
				int left = ((Integer)tree.phrase[tree.phrase.length-3].attribute).intValue();
				int right = ((Integer)tree.phrase[tree.phrase.length-2].attribute).intValue();
				tree.attribute = new Integer(left + right);
			}
		};
		Semantics multiply = new Semantics() {
			public void evaluate(ParseTree tree) {
				int left = ((Integer)tree.phrase[tree.phrase.length-3].attribute).intValue();
				int right = ((Integer)tree.phrase[tree.phrase.length-2].attribute).intValue();
				tree.attribute = new Integer(left * right);
			}
		};
		Semantics decode = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = Integer.decode((String)tree.phrase[0].attribute);
			}
		};
		Semantics second = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = tree.phrase[1].attribute;
			}
		};
		Semantics zero = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = new Integer(0);
			}
		};
		Semantics one = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = new Integer(1);
			}
		};

		/**
		* Syntax Specification
		*/
		put("Start", new Object[][] {
			{"Expression", print}
		});
		put("Expression", new Object[][] {
			{"Term", "Expression->", add}
		});
		put("Expression->", new Object[][] {
			{"+", "Term", "Expression->", add},
			{zero}
		});
		put("Term", new Object[][] {
			{"Factor", "Term->", multiply}
		});
		put("Term->", new Object[][] {
			{"*", "Factor", "Term->", multiply},
			{one}
		});
		put("Factor", new Object[][] {
			{"LITERAL", decode},
			{"(", "Expression", ")", second}
		});
	}

	public static void main(String[] argv) throws Exception {
		new ArithmeticLL().interpret(argv);
	}
}
