import gi.*;

class CodeGenerator extends LL1_Grammar {

	CodeGenerator() {

		/**
		* Lexical Specification
		*/
		int INFINITY = -1;

		put("LITERAL", new Repetition(PosixClass.digit(), 1, INFINITY)); // [[:digit:]]+
		put("IDENTIFIER", new Repetition(PosixClass.alpha(), 1, INFINITY)); // [[:alpha:]]+
		put("SPACE", new Repetition(PosixClass.space(), 1, INFINITY)); // [[:space:]]+

		/**
		* Semantic Specification
		*/
		Semantics store = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println("      STORE " + tree.phrase[0].attribute);
			}
		};
		Semantics label_loop = new Semantics() {
			int loopLabel = 0;

			public void evaluate(ParseTree tree) {
				System.out.println("top" + ++loopLabel + ":");
				tree.attribute = new Integer(loopLabel);
			}
		};
		Semantics exit_loop = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println(tree.attribute);
			}
		};
		Semantics go_loop = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println("      GOTO  top" + tree.attribute);
				System.out.println("end" + tree.attribute + ":");
			}
		};
		Semantics compare = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println("      COMP");
				System.out.print("      BGE   end");
			}
		};
		Semantics add = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println("      ADD");
			}
		};
		Semantics multiply = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println("      MULT");
			}
		};
		Semantics load_id = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println("      LOAD  " + tree.phrase[0].attribute);
			}
		};
		Semantics load_literal = new Semantics() {
			public void evaluate(ParseTree tree) {
				System.out.println("      LC    " + tree.phrase[0].attribute);
			}
		};

		/**
		* Syntax Specification
		*/
		put("StmtList", new Object[][] {
			{"Stmt", "StmtList"},
			{}
		});
		put("Stmt", new Object[][] {
			{"IDENTIFIER", ":=", "Expr", store, ";"},
			{"while", label_loop, "Condition", exit_loop, "loop", "StmtList", go_loop, "end", "loop", ";"}
		});
		put("Condition", new Object[][] {
			{"Expr", "<", "Expr", compare}
		});
		put("Expr", new Object[][] {
			{"Term", "Expr->"}
		});
		put("Expr->", new Object[][] {
			{"+", "Term", add, "Expr->"},
			{}
		});
		put("Term", new Object[][] {
			{"Factor", "Term->"}
		});
		put("Term->", new Object[][] {
			{"*", "Factor", multiply, "Term->"},
			{}
		});
		put("Factor", new Object[][] {
			{"IDENTIFIER", load_id},
			{"LITERAL", load_literal},
			{"(", "Expr", ")"}
		});
	}

	public static void main(String[] argv) throws Exception {
		new CodeGenerator().interpret(argv);
	}
}
