import gi.*;
import java.util.*;

class TypeChecker extends LL1_Grammar {

	TypeChecker() throws Exception {

		/**
		* Lexical Specification
		*/
		put("IDENTIFIER", Lexicon.expression("[[:alpha:]][[:alnum:]]*"));
		put("LITERAL", Lexicon.expression("[[:digit:]]+|true|false"));
		put("RELOP", Lexicon.expression("=|<|>|<>|<=|>="));
		put("ADDOP", Lexicon.expression("\\+|-|or"));
		put("MULOP", Lexicon.expression("\\*|div|mod|and"));
		put("SPACE", Lexicon.expression("[[:space:]]+"));
		put("COMMENT", Lexicon.expression("\\{[^}]*\\}"));

		/**
		* Semantic Specification
		*/
		final Map symbols = new HashMap();

		Semantics declareProgram = new Semantics() {
			public void evaluate(ParseTree tree) {
				symbols.put(tree.phrase[1].attribute, "program");
			}
		};
		Semantics declareVariable= new Semantics() {
			public void evaluate(ParseTree tree) throws Exception {
				ParseTree[] phrase = tree.phrase;
				String identifier = (String)phrase[phrase.length-3].attribute;
				String type = (String)phrase[phrase.length-2].attribute;
				String existingType;
				if ((existingType = (String)symbols.put(identifier, type)) != null)
					throw new Exception("variable " + identifier + " already declared to be type " + existingType);
			}
		};
		Semantics first = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = tree.phrase[0].attribute;
			}
		};
		Semantics second = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = tree.phrase[1].attribute;
			}
		};
		Semantics checkAssignment = new Semantics() {
			public void evaluate(ParseTree tree) throws Exception {
				String variableType = (String)symbols.get(tree.phrase[0].attribute);
				String expressionType = (String)tree.phrase[2].attribute;
				if (!expressionType.equals(variableType))
					throw new Exception("variable undeclared or of wrong type");
			}
		};
		Semantics checkCondition = new Semantics() {
			public void evaluate(ParseTree tree) throws Exception {
				String type = (String)tree.phrase[1].attribute;
				if (!type.equals("boolean"))
					throw new Exception("condition expression must be boolean");
			}
		};
		Semantics checkRelop = new Semantics() {
			public void evaluate(ParseTree tree) throws Exception {
				String leftType = ((String [])tree.attribute)[0];
				String rightType = (String)tree.phrase[1].attribute;
				if (!(leftType.equals("integer") && rightType.equals("integer")))
					throw new Exception("relational operands must be integer");
				((String [])tree.attribute)[1] = "boolean";
			}
		};
		Semantics checkSign = new Semantics() {
			public void evaluate(ParseTree tree) throws Exception {
				String sign = (String)tree.phrase[0].attribute;
				String type = (String)tree.phrase[1].attribute;
				if (sign.equals("or"))
					throw new Exception("or is not a valid sign operator");
				else if (!type.equals("integer"))
					throw new Exception("sign operand must be integer");
				tree.phrase[3].attribute = new String[]{type, null};
			}
		};
		Semantics checkAddop = new Semantics() {
			public void evaluate(ParseTree tree) throws Exception {
				String leftType = ((String [])tree.attribute)[0];
				String addop = (String)tree.phrase[0].attribute;
				String rightType = (String)tree.phrase[1].attribute;
				if (addop.equals("or") && !(leftType.equals("boolean") && rightType.equals("boolean")))
					throw new Exception("or operands must be boolean");
				else if (!addop.equals("or") && !(leftType.equals("integer") && rightType.equals("integer")))
					throw new Exception("arithmetic operands must be integer");
				tree.phrase[3].attribute = new String[]{leftType, null};
			}
		};
		Semantics checkMulop = new Semantics() {
			public void evaluate(ParseTree tree) throws Exception {
				String leftType = ((String [])tree.attribute)[0];
				String mulop = (String)tree.phrase[0].attribute;
				String rightType = (String)tree.phrase[1].attribute;
				if (mulop.equals("and") && !(leftType.equals("boolean") && rightType.equals("boolean")))
					throw new Exception("and operands must be boolean");
				else if (!mulop.equals("and") && !(leftType.equals("integer") && rightType.equals("integer")))
					throw new Exception("arithmetic operands must be integer");
				tree.phrase[3].attribute = new String[]{leftType, null};
			}
		};
		Semantics checkNot = new Semantics() {
			public void evaluate(ParseTree tree) throws Exception {
				String type = (String)tree.phrase[1].attribute;
				if (!type.equals("boolean"))
					throw new Exception("not operand must be boolean");
				tree.attribute = "boolean";
			}
		};
		Semantics inheritType = new Semantics() {
			public void evaluate(ParseTree tree) {
				String type = (String)tree.phrase[0].attribute;
				tree.phrase[2].attribute = new String[]{type, null};
			}
		};
		Semantics synthesizePassed = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = ((String [])tree.phrase[2].attribute)[1];
			}
		};
		Semantics passInherited = new Semantics() {
			public void evaluate(ParseTree tree) {
				((String [])tree.attribute)[1] = ((String [])tree.attribute)[0];
			}
		};
		Semantics passPassed = new Semantics() {
			public void evaluate(ParseTree tree) {
				((String [])tree.attribute)[1] = ((String [])tree.phrase[3].attribute)[1];
			}
		};
		Semantics identifierType = new Semantics() {
			public void evaluate(ParseTree tree) {
				tree.attribute = symbols.get(tree.phrase[0].attribute);
			}
		};
		Semantics literalType = new Semantics() {
			public void evaluate(ParseTree tree) {
				String literal = (String)tree.phrase[0].attribute;
				tree.attribute = (literal.equals("true") || literal.equals("false"))
					? "boolean"
					: "integer";
			}
		};

		/**
		* Syntax Specification
		*/
		put("Program", new Object[][]{
			{"program", "IDENTIFIER", declareProgram, ";", "Decls", "CompoundStmt", "."}
		});
		put("Decls", new Object[][]{
			{"var", "DeclList"},
			{}
		});
		put("DeclList", new Object[][]{
			{"Decl", "DeclList->"}
		});
		put("DeclList->", new Object[][]{
			{"Decl", "DeclList->"},
			{}
		});
		put("Decl", new Object[][]{
			{"IDENTIFIER", "IdList", declareVariable}
		});
		put("IdList", new Object[][]{
			{",", "IDENTIFIER", "IdList", declareVariable},
			{":", "Type", second, ";"}
		});
		put("Type", new Object[][]{
			{"integer", first},
			{"boolean", first}
		});
		put("CompoundStmt", new Object[][]{
			{"begin", "StmtList", "end"}
		});
		put("StmtList", new Object[][]{
			{"Stmt", "StmtList->"}
		});
		put("StmtList->", new Object[][]{
			{";", "Stmt", "StmtList->"},
			{}
		});
		put("Stmt", new Object[][]{
			{"Variable", ":=", "Expr", checkAssignment},
			{"CompoundStmt"},
			{"if", "Expr", checkCondition, "then", "Stmt", "Stmt->"},
			{"while", "Expr", checkCondition, "do", "Stmt"}
		});
		put("Stmt->", new Object[][]{
			{},
			{"else", "Stmt"}
		});
		put("Variable", new Object[][]{
			{"IDENTIFIER", first}
		});
		put("Expr", new Object[][]{
			{"SimpleExpr", inheritType, "Expr->", synthesizePassed}
		});
		put("Expr->", new Object[][]{
			{"RELOP", "SimpleExpr", checkRelop},
			{passInherited}
		});
		put("SimpleExpr", new Object[][]{
			{"Term", inheritType, "SimpleExpr->", synthesizePassed},
			{"Sign", "Term", checkSign, "SimpleExpr->", synthesizePassed}
		});
		put("SimpleExpr->", new Object[][]{
			{"ADDOP", "Term", checkAddop, "SimpleExpr->", passPassed},
			{passInherited}
		});
		put("Term", new Object[][]{
			{"Factor", inheritType, "Term->", synthesizePassed}
		});
		put("Term->", new Object[][]{
			{"MULOP", "Factor", checkMulop, "Term->", passPassed},
			{passInherited}
		});
		put("Factor", new Object[][]{
			{"IDENTIFIER", identifierType},
			{"LITERAL", literalType},
			{"(", "Expr", second, ")"},
			{"not", "Factor", checkNot}
		});
		put("Sign", new Object[][]{
			{"ADDOP", first}
		});
	}

	public static void main(String[] argv) throws Exception {
		ParseTree tree = new TypeChecker().interpret(argv);
		if (tree != null) System.out.println(tree);
	}
}
